const fs = require("node:fs");
const PccApp = require("../../../picockpit-library/picockpit-base-app");

class Editor extends PccApp {
  constructor() {
    super("editor");
    this.client.on("message", (topic, message) => {
      this.handleMessage(
        this.simplifyTopic(topic),
        this.formatMessage(message),
      );
    });
  }

  simplifyTopic = (topic) => {
    let temp = topic.split("/");
    return temp[temp.length - 1];
  };

  formatMessage = (message) => {
    return JSON.parse(message.toString());
  };

  handleMessage = (topic, message) => {
    let publishTo;
    let messageObject;

    this.logger.debug(topic);
    this.logger.debug(message);
    if (this.config.disable) {
      let returnObject = { path: message.path };
      returnObject["error"] = true;
      returnObject["error_message"] =
        "The editor was disabled inside the configuration in /etc/picockpit-client. Please re-enable it there if you want to use it.";
      returnObject["content"] = [];
      publishTo = "client/send_folder_content";
      this.publish(publishTo, returnObject);
    } else {
      switch (topic) {
        case "read_file":
          messageObject = this.readFile(message);
          publishTo = "client/send_file_content";
          break;
        case "read_folder":
          messageObject = this.readFolder(message);
          publishTo = "client/send_folder_content";
          break;
        case "create_folder":
          messageObject = this.createFolder(message);
          publishTo = "client/create_folder_result";
          break;
        case "write_file":
          messageObject = this.writeFile(message);
          publishTo = "client/write_file_result";
          break;
        default:
          throw new Error(`Wrong message reached editor app? Topic: ${topic}`);
      }
      this.publish(publishTo, messageObject);
    }
  };

  createFolder = (message) => {
    const { path, createParents } = message;
    let returnObject = { path, error: false, error_message: "" };

    try {
      fs.mkdirSync(path, { recursive: createParents });
    } catch (err) {
      returnObject["error"] = true;
      returnObject["error_message"] = err;
    }

    return returnObject;
  };

  readFile = (message) => {
    const { path } = message;
    let returnObject = { path, error: false, error_message: "" };

    try {
      returnObject["content"] = fs.readFileSync(path).toString();
    } catch (err) {
      returnObject["error"] = true;
      returnObject["error_message"] = err;
    }
    return returnObject;
  };

  readFolder = (message) => {
    const { path } = message;
    let returnObject = { path, error: false, error_message: "" };

    try {
      const unformattedContent = fs.readdirSync(path, { withFileTypes: true });
      const content = [];
      for (const item of unformattedContent) {
        var tempPath;
        if (path == "/") {
          tempPath = `/${item["name"]}`;
        } else {
          tempPath = `${path}/${item["name"]}`;
        }
        var size;
        try {
          size = fs.statSync(tempPath).size;
        } catch (err) {
          // Handle error
          console.error("Error getting file size:", err);
          return null;
        }

        var too_big_or_binary = false;
        if (size > 1000000) {
          too_big_or_binary = true;
        }

        content.push({
          path: tempPath,
          name: item["name"],
          type: item.isDirectory() ? "dir" : "file",
          too_big_or_binary: too_big_or_binary, // TODO: Add this
        });
      }
      returnObject["content"] = content;
    } catch (err) {
      returnObject["error"] = true;
      returnObject["error_message"] = err;
    }

    return returnObject;
  };

  writeFile = (message) => {
    const { path, content, create_file } = message;
    let returnObject = { path, error: false, error_message: "" };

    if (!create_file && !fs.existsSync(path)) {
      returnObject["error"] = true;
      returnObject["error_message"] = "no_permission_to_create_file";
    } else {
      try {
        fs.writeFileSync(path, content);
      } catch (err) {
        returnObject["error"] = true;
        returnObject["error_message"] = err;
      }
    }

    return returnObject;
  };

  // Not used currently

  /* isExecutable = (path) => {
        const executable =
            execSync(
                `if [ -x ${path} ]; then echo "true"; else echo "false"; fi`
            ) === "true" 
        return executable
    } */
}

const editor = new Editor();
